const stringEnumerator = function(array){
	var i = 0;
	
	this.hasMore = function(){
		if(i < array.length){
			return true;
		} else {
			return false;
		}
	};
	
	this.getNext = function(){
		return array[i++];
	};
};

const countryLookup = new function(){
	var lookupDatabaseString = '';
	var lookupDatabase = new Array(5);
	var temp = new Array();
	var databaseParsedUntil = 0;
	var databaseReadyLoading = false;
	
	var _locale;
	
	const loadLookupDatabase = function(){
		var req = new XMLHttpRequest();
		req.open('GET', 'chrome://countrylookup/content/ip-to-country.csv', true);
		
		req.onreadystatechange = function(aEvt){
			if(req.readyState == 4){
				lookupDatabaseString = req.responseText;
				parseLookupDatabaseString();
				countryLookup.parseLookupDatabase();
			}
		};
		
		req.send(null);
	};
	
	const constructLookupDatabase = function(){
		for(var i = 0; i <= 4; i++){
			lookupDatabase[i] = new Array(10);
			for(var j = 0; j <= 10; j++){
				lookupDatabase[i][j] = new Array(10);
				for(var k = 0; k <= 10; k++){
					lookupDatabase[i][j][k] = new Array(10);
				}
			}
		}
	};
	
	const parseLookupDatabaseString = function(){
		temp = lookupDatabaseString.split('\r\n');
	};
	
	this.parseLookupDatabase = function(){
		
		/** LookupDatabase format: ******************************
		 *                                                      *
		 *       [0]\            0 -   999 999 999              *
		 *           \[0][0]\          0 -  99 999 999          *
		 *                   \[0][0][0]          0 -  9 999 999 *
		 *                    [0][0][1] 10 000 000 - 19 999 999 *
		 *                    [0][0][2] 20 000 000 - 29 999 999 *
		 *                    [0][0][3] 30 000 000 - 39 999 999 *
		 *            [0][1] 100 000 000 - 199 999 999          *
		 *            [0][2] 200 000 000 - 299 999 999          *
		 *            [0][3] 300 000 000 - 399 999 999          *
		 *       [1] 1 000 000 000 - 1 999 999 999              *
		 *       [2] 2 000 000 000 - 2 999 999 999              *
		 *       [3] 3 000 000 000 - 3 999 999 999              *
		 *       [4] 4 000 000 000 - 4 294 967 295              *
		 *                                                      *
		 ********************************************************/
			
		// E.g., 3 212 000 000 - 6 132 000 000 has to go in categories:
		// [3][2][1] to [6][1][3]
		//
		// [ 3 ][ 2 ][1-9] C
		// [ 3 ][3-9][ * ] B
		// [4-5][ * ][ * ] A
		// [ 6 ][0-0][ * ] B
		// [ 6 ][ 1 ][0-3] C
		//
		// [      scat1      ][      scat2      ][scat3-  9  ] Cs
		// [      scat1      ][scat2+1 -    9   ][     *     ] Bs
		// [scat1+1 - ecat1-1][        *        ][     *     ] A
		// [      ecat1      ][    0   - ecat1-1][     *     ] Be
		// [      ecat1      ][      ecat1      ][  0  -ecat3] Ce
		
		while(databaseParsedUntil < temp.length){
			thisArray = temp[databaseParsedUntil].substring(1, temp[databaseParsedUntil].length-1).split('","');
			
			var scat1 = thisArray[0];
			var ecat1 = thisArray[1];
			
			var scat2 = scat1 % 1000000000;
			var ecat2 = ecat1 % 1000000000;
			
			scat1 = (scat1 - scat2) / 1000000000;
			ecat1 = (ecat1 - ecat2) / 1000000000;
			
			var scat3 = scat2 % 100000000;
			var ecat3 = ecat2 % 100000000;
			
			scat2 = (scat2 - scat3) / 100000000;
			ecat2 = (ecat2 - ecat3) / 100000000;
			
			var scat4 = scat3 % 10000000;
			var ecat4 = ecat3 % 10000000;
			
			scat3 = (scat3 - scat4) / 10000000;
			ecat3 = (ecat3 - ecat4) / 10000000;
			
			// A
			for(var j = scat1+1; j <= ecat1-1; j++){
				for(k in lookupDatabase[j]){
					for(l in lookupDatabase[j][k]){
						lookupDatabase[j][k][l][lookupDatabase[j][k][l].length] = thisArray;
					}
				}
			}
			
			// Bs
			if(scat1 == ecat1){
				end = ecat2-1;
			} else {
				end = 9;
			}
			for(var j = scat2+1; j <= end; j++){
				for(k in lookupDatabase[scat1][j]){
					lookupDatabase[scat1][j][k][lookupDatabase[scat1][j][k].length] = thisArray;
				}
			}
			
			// Be
			if(scat1 == ecat1){
				// We already did it above
			} else {
				for(var j = 0; j <= ecat2-1; j++){
					for(k in lookupDatabase[ecat1][j]){
						lookupDatabase[ecat1][j][k][lookupDatabase[ecat1][j][k].length] = thisArray;
					}
				}
			}
			
			// Cs
			if(scat1 == ecat1 && scat2 == ecat2){
				end = ecat3;
			} else {
				end = 9;
			}
			for(var j = scat3; j <= end; j++){
				lookupDatabase[scat1][scat2][j][lookupDatabase[scat1][scat2][j].length] = thisArray;
			}
			
			// Ce
			if(scat1 == ecat1 && scat2 == ecat2){
				// We already did it above
			} else {
				for(var j = 0; j <= ecat3; j++){
					lookupDatabase[ecat1][ecat2][j][lookupDatabase[ecat1][ecat2][j].length] = thisArray;
				}
			}
			
			databaseParsedUntil++;
			
			if(databaseParsedUntil % 50 == 0){
				// Try again
				dispRoute();
				// Take a break
				setTimeout("countryLookup.parseLookupDatabase();", 0);
				break;
			}
		}
		
		if(databaseParsedUntil == temp.length){
			temp = null;
			lookupDatabaseString = null;
			databaseParsedUntil = null;
			databaseReadyLoading = true;
			dispRoute();
		}
	};
	
	var observer = new function(){
		this.init = function(){
			register();
		};
		
		var register = function(){
			var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
			observerService.addObserver(observer, "countryLookupDatabaseReady", false);
		};
		
		this.observe = function(subject, topic, data){
			lookupDatabase = subject;
		};
	};
	
	const capitalize = function(name){
		const insertCapitalAfter = function(word, character){
			word = word.split(character);
			
			for(var i in word){
				word[i] = word[i].charAt(0).toUpperCase() + word[i].substring(1);
			}
			
			word = word.join(character);
			
			return word;
		};
		
		name = name.toLowerCase();
		name = insertCapitalAfter(name, " ");
		name = insertCapitalAfter(name, "-");
		
		return name;
	};
	
	const inet_aton = function(address){
		address = address.split(".");
		
		number = address[0]*256*256*256 + address[1]*256*256 + address[2]*256 + Number(address[3]);
		
		return number;
	};
	
	const lookupIP = function(address){
		address = inet_aton(address);
		
		countryCode = "UKN";
		countryName = _locale.getString("Unknown");
		
		var cat1 = address;
		
		var cat2 = cat1 % 1000000000;		
		cat1 = (cat1 - cat2) / 1000000000;
		
		var cat3 = cat2 % 100000000;		
		cat2 = (cat2 - cat3) / 100000000;
		
		var cat4 = cat3 % 10000000;		
		cat3 = (cat3 - cat4) / 10000000;
		
		for(var i in lookupDatabase[cat1][cat2][cat3]){
			if(lookupDatabase[cat1][cat2][cat3][i][0] <= address && address <= lookupDatabase[cat1][cat2][cat3][i][1]){
				countryCode = lookupDatabase[cat1][cat2][cat3][i][3];
				countryName = lookupDatabase[cat1][cat2][cat3][i][4];
			}
		}
		
		return new function(){
			this.code = countryCode;
			this.name = countryName;
		};
	};
	
	const isPrivate = function(address){
		// Is the supplied IP address part of a private network?
		// http://www.iana.org/faqs/abuse-faq.htm#SpecialUseAddresses
		var number = inet_aton(address);
		
		if((167772160 <= number && number <= 184549375)
		|| (2886729728 <= number && number <= 2887778303)
		|| (3232235520 <= number && number <= 3232301055)){
			return true;
		} else {
			return false;
		}
	};
	
	const dispRoute = function(){
		var originIP = null;
		
		var receivedHeaders = new Array();
		var IPArray = new Array();
		
		// Look for Received header
		for (var i in currentHeaderData) {
			if (currentHeaderData[i].headerName.toLowerCase() == "received") {
				receivedHeaders[receivedHeaders.length] = currentHeaderData[i].headerValue;
			}
		}
		
		for(var i = receivedHeaders.length-1; i>=0; i--){
			var currentIPs = receivedHeaders[i].match(/\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/g);
			if(currentIPs != null){
				IPArray = IPArray.concat(currentIPs);
			}
		}
		
		xOriginatingIP = null;
		
		// Look for X-Originating-IP header
		for (var i in currentHeaderData) {
            if (currentHeaderData[i].headerName.toLowerCase() == "x-originating-ip") {
				xOriginatingIP = currentHeaderData[i].headerValue;
				if(xOriginatingIP.charAt(0) == "["){
					xOriginatingIP = xOriginatingIP.substring(1, xOriginatingIP.length-1)
				}
            }
        }
		
		// If X-Originating-IP isn't set, look at Received headers in any case
		// If X-Originating-IP is also in the Received headers,
		// the Received headers know better.
		// Else, it almost certainly is from a web-based service,
		// which inserted an X-Originating-IP header, like Hotmail.
		// If X-Originating-IP is private, it's from an Intranet web service,
		// and of no use to us.
		if(xOriginatingIP != null && IPArray.indexOf(xOriginatingIP) == -1 && !isPrivate(xOriginatingIP)){
			originIP = xOriginatingIP;
		} else {
			for(var i in IPArray){
				// Private network IPs are of no use to us!
				if(!isPrivate(IPArray[i])){
					originIP = IPArray[i];
				}
			}
		}
		
		if(originIP == null){
			// Maybe it's an NNTP message
			for (var i in currentHeaderData) {
				if (currentHeaderData[i].headerName.toLowerCase() == "nntp-posting-host") {
					originIP = currentHeaderData[i].headerValue;
				}
			}
		}
        
        if(originIP == null){
			var country = new function(){
				this.code = "UKN";
				this.name = _locale.getString("Unknown");
			};
        } else {
			var country = lookupIP(originIP);
        }
        
        country.code = country.code.toLowerCase();
        country.name = capitalize(country.name);
        
        if(!databaseReadyLoading && country.code == "ukn" && originIP != null){
			document.getElementById("countryLookupIcon").width  = "20px";
			document.getElementById("countryLookupIcon").height = "20px";
			document.getElementById("countryLookupIcon").src    = "chrome://messenger/skin/icons/throbber.gif";
			document.getElementById("countryLookupIcon").setAttribute("tooltiptext", _locale.getString("Loading"));
        } else {
			document.getElementById("countryLookupIcon").width  = "48px";
			document.getElementById("countryLookupIcon").height = "48px";
			document.getElementById("countryLookupIcon").src    = "chrome://countrylookup/content/flags/" + country.code + ".png";
			document.getElementById("countryLookupIcon").setAttribute("tooltiptext", country.name);
        }
	};

	this.init = function(){
		
		_locale = document.getElementById('countryLookupStrings');
		
		constructLookupDatabase();
		loadLookupDatabase();
		
		const listener = new function(){
			this.onStartHeaders	= function(){};
			this.onEndHeaders	= dispRoute;
		};

		gMessageListeners.push(listener);
	};
	
}

addEventListener('messagepane-loaded', countryLookup.init, true);